@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,8 +20,19 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 
-#if defined(HAVE_SEARCH_H)
+#if defined(HAVE_SEARCH_H) && 	\
+     defined(HAVE_HSEARCH)
 #include <search.h>
+#else
+typedef enum {
+	FIND,
+	ENTER
+} ACTION;
+
+typedef struct {
+	char *key;
+	void *data;
+} ENTRY;
 #endif
 
 #define MIN_HSEARCH_SIZE	(1 * KB)
@@ -46,14 +57,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		  NULL }
 };
 
-#if !(defined(HAVE_SEARCH_H) ||	\
-     defined(HAVE_HSEARCH))
-typedef struct {
-	char *key;
-	void *data;
-} ENTRY;
-#endif
-
 typedef struct {
 	uint32_t hash;
 	ENTRY entry;
@@ -70,7 +73,7 @@ static int hcreate_nonlibc(size_t nel)
 		if (stress_is_prime64((uint64_t)nel))
 			break;
 	}
-	htable = calloc(nel, sizeof(*htable));
+	htable = (hash_table_t *)calloc(nel, sizeof(*htable));
 	if (!htable) {
 		htable_size = 0;
 		errno = ENOMEM;
@@ -139,44 +142,15 @@ static const stress_hsearch_method_t stress_hsearch_methods[] = {
 	{ "hsearch-nonlibc",	hcreate_nonlibc, hsearch_nonlibc, hdestroy_nonlibc },
 };
 
-
-/*
- *  stress_set_hsearch_size()
- *      set hsearch size from given option string
- */
-static int stress_set_hsearch_size(const char *opt)
+static const char *stress_hsearch_method(const size_t i)
 {
-	uint64_t hsearch_size;
-
-	hsearch_size = stress_get_uint64(opt);
-	stress_check_range("hsearch-size", hsearch_size,
-		MIN_HSEARCH_SIZE, MAX_HSEARCH_SIZE);
-	return stress_set_setting("hsearch-size", TYPE_ID_UINT64, &hsearch_size);
+	return (i < SIZEOF_ARRAY(stress_hsearch_methods)) ? stress_hsearch_methods[i].name : NULL;
 }
 
-static int stress_set_hsearch_method(const char *opt)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(stress_hsearch_methods); i++) {
-		if (strcmp(opt, stress_hsearch_methods[i].name) == 0) {
-			stress_set_setting("hsearch-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "ssearch-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(stress_hsearch_methods); i++) {
-		(void)fprintf(stderr, " %s", stress_hsearch_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-	return -1;
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_hsearch_method,	stress_set_hsearch_method },
-	{ OPT_hsearch_size,	stress_set_hsearch_size },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_hsearch_method, "hsearch-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_hsearch_method },
+	{ OPT_hsearch_size,   "hsearch-size",   TYPE_ID_UINT64, MIN_HSEARCH_SIZE, MAX_HSEARCH_SIZE, NULL },
+	END_OPT,
 };
 
 /*
@@ -187,7 +161,7 @@ static int OPTIMIZE3 stress_hsearch(stress_args_t *args)
 {
 	uint64_t hsearch_size = DEFAULT_HSEARCH_SIZE;
 	size_t i, max;
-	int ret = EXIT_FAILURE;
+	int rc = EXIT_FAILURE;
 	char **keys;
 	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 	hsearch_func_t hsearch_func;
@@ -214,7 +188,7 @@ static int OPTIMIZE3 stress_hsearch(stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	keys = calloc(max, sizeof(*keys));
+	keys = (char **)calloc(max, sizeof(*keys));
 	if (!keys) {
 		pr_err("%s: cannot allocate keys\n", args->name);
 		goto free_hash;
@@ -241,21 +215,27 @@ static int OPTIMIZE3 stress_hsearch(stress_args_t *args)
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	rc = EXIT_SUCCESS;
 	do {
-		for (i = 0; stress_continue_flag() && (i < max); i++) {
-			ENTRY e, *ep;
+		for (i = 0; LIKELY(stress_continue_flag() && (i < max)); i++) {
+			ENTRY e;
+			const ENTRY *ep;
 
 			e.key = keys[i];
 			e.data = NULL;	/* Keep Coverity quiet */
 			ep = hsearch_func(e, FIND);
 			if (verify) {
-				if (ep == NULL) {
+				if (UNLIKELY(ep == NULL)) {
 					pr_fail("%s: cannot find key %s\n", args->name, keys[i]);
+					rc = EXIT_FAILURE;
 				} else {
-					if (i != (size_t)ep->data) {
+					if (UNLIKELY(i != (size_t)ep->data)) {
 						pr_fail("%s: hash returned incorrect data %zd\n", args->name, i);
+						rc = EXIT_FAILURE;
 					}
 				}
 			}
@@ -263,8 +243,6 @@ static int OPTIMIZE3 stress_hsearch(stress_args_t *args)
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
-	ret = EXIT_SUCCESS;
-
 free_all:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -288,13 +266,13 @@ free_hash:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	hdestroy_func();
 
-	return ret;
+	return rc;
 }
 
-stressor_info_t stress_hsearch_info = {
+const stressor_info_t stress_hsearch_info = {
 	.stressor = stress_hsearch,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SEARCH,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
