@@ -188,7 +188,7 @@ static void stress_open_max(const char *opt_name, const char *opt_arg, stress_ty
 
 static const stress_opt_t opts[] = {
 	{ OPT_open_fd,  "open-fd",  TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_open_max,	"open-max", TYPE_ID_CALLBACK, 0, 0, stress_open_max },
+	{ OPT_open_max,	"open-max", TYPE_ID_CALLBACK, 0, 1, stress_open_max },
 	END_OPT,
 };
 
@@ -909,8 +909,9 @@ static int open_rdonly_trunc(
 
 	/* undefined behaviour, will open, may truncate on some systems */
 	fd = open_arg2(filename, O_RDONLY | O_TRUNC, duration, count);
-	if (fd >= 0)
-		(void)shim_unlink(filename);
+	(void)fd;
+	(void)shim_unlink(filename);
+
 	return fd;
 }
 #endif
@@ -1033,7 +1034,7 @@ static stress_open_func_t open_funcs[] = {
 static void stress_fd_dir(const char *path, double *duration, double *count)
 {
 	for (;;) {
-		struct dirent *de;
+		const struct dirent *de;
 		DIR *dir;
 
 		dir = opendir(path);
@@ -1083,7 +1084,10 @@ static int stress_open(stress_args_t *args)
 		return stress_exit_status(-ret);
 	(void)stress_temp_dir_args(args, temp_dir, sizeof(temp_dir));
 
-	(void)stress_get_setting("open-max", &open_max);
+	if (!stress_get_setting("open-max", &open_max)) {
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			open_max = 1;
+	}
 	(void)stress_get_setting("open-fd", &open_fd);
 
 	/* Limit to maximum size_t allocation size */
@@ -1095,7 +1099,6 @@ static int stress_open(stress_args_t *args)
 	if (open_max < 1)
 		open_max = 1;
 
-
 	sz = open_max * sizeof(*fds);
 	fds = (int *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
