@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,11 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-killpid.h"
+
+#define MIN_POLL_FDS	(1)
+#define MAX_POLL_FDS	(8192)	/* Must be never larger than 65535 */
 
 #if defined(HAVE_SYS_SELECT_H)
 #include <sys/select.h>
@@ -28,30 +33,23 @@
 #endif
 
 static const stress_help_t help[] = {
-	{ "P N", "poll N",	"start N workers exercising zero timeout polling" },
-	{ NULL, "poll-fds N",	"use N file descriptors" },
-	{ NULL,	"poll-ops N",	"stop after N poll bogo operations" },
+	{ "P N", "poll N",	    "start N workers exercising zero timeout polling" },
+	{ NULL, "poll-fds N",	    "use N file descriptors" },
+	{ NULL,	"poll-ops N",	    "stop after N poll bogo operations" },
+	{ NULL, "poll-random-us N", "use random poll delays of 0..N-1 microseconds" },
 	{ NULL,	NULL,		NULL }
 };
 
-static int stress_set_poll_fds(const char *opt)
-{
-	size_t max_fds;
-
-        max_fds = (size_t)stress_get_uint32(opt);
-        stress_check_range("poll-fds", (uint64_t)max_fds, 1, 8192);
-        return stress_set_setting("poll-fds", TYPE_ID_SIZE_T, &max_fds);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_poll_fds,		stress_set_poll_fds },
-	{ 0,			NULL },
+static const stress_opt_t opts[] = {
+	{ OPT_poll_fds,       "poll-fds",       TYPE_ID_SIZE_T, MIN_POLL_FDS, MAX_POLL_FDS, NULL },
+	{ OPT_poll_random_us, "poll-random-us", TYPE_ID_UINT32, 0, 1000000, NULL },
+	END_OPT,
 };
 
-#if defined(HAVE_POLL_H)
+#if defined(HAVE_POLL_H) &&	\
+    defined(HAVE_POLL)
 
 #define MAX_PIPES	(5)
-#define POLL_BUF	(4)
 
 typedef struct {
 	int fd[2];
@@ -61,33 +59,30 @@ typedef struct {
  *  pipe_read()
  *	read a pipe with some verification and checking
  */
-static ssize_t pipe_read(const stress_args_t *args, const int fd, const size_t n)
+static ssize_t OPTIMIZE3 pipe_read(stress_args_t *args, const int fd, const size_t n)
 {
-	while (keep_stressing_flag()) {
-		ssize_t ret;
-		char buf[POLL_BUF];
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
-		ret = read(fd, buf, sizeof(buf));
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
-			if (ret < 0) {
+	while (stress_continue_flag()) {
+		ssize_t ret;
+		uint16_t buf ALIGN64 = ~0;
+
+		ret = read(fd, &buf, sizeof(buf));
+		if (UNLIKELY(verify)) {
+			if (LIKELY(ret > 0)) {
+				if (UNLIKELY(buf != (uint16_t)n)) {
+					pr_fail("%s: pipe read error, "
+						"expecting different data on "
+						"pipe\n", args->name);
+					return ret;
+				}
+			} else if (UNLIKELY(ret < 0)) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					continue;
 				pr_fail("%s: pipe read error detected, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				return ret;
 			}
-			if (ret > 0) {
-				ssize_t i;
-
-				for (i = 0; i < ret; i++) {
-					if (buf[i] != (int)('0' + n)) {
-						pr_fail("%s: pipe read error, "
-							"expecting different data on "
-							"pipe\n", args->name);
-						return ret;
-					}
-				}
-			}
 		}
 		return ret;
 	}
@@ -98,23 +93,32 @@ static ssize_t pipe_read(const stress_args_t *args, const int fd, const size_t n
  *  stress_poll()
  *	stress system by rapid polling system calls
  */
-static int stress_poll(const stress_args_t *args)
+static int OPTIMIZE3 stress_poll(stress_args_t *args)
 {
 	pid_t pid;
-	int rc = EXIT_SUCCESS;
-	size_t i, max_fds = MAX_PIPES;
+	int rc = EXIT_SUCCESS, parent_cpu;
+	register size_t i;
+	size_t max_fds = MAX_PIPES, max_rnd_fds;
 	pipe_fds_t *pipe_fds;
 	struct pollfd *poll_fds;
+	int *rnd_fds_index;
+	uint32_t poll_random_us = 0;
 
-	(void)stress_get_setting("poll-fds", &max_fds);
+	if (!stress_get_setting("poll-fds", &max_fds)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			max_fds = MAX_POLL_FDS;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			max_fds = MIN_POLL_FDS;
+	}
+	(void)stress_get_setting("poll-random-us", &poll_random_us);
 
-	pipe_fds = calloc((size_t)max_fds, sizeof(*pipe_fds));
+	pipe_fds = (pipe_fds_t *)calloc(max_fds, sizeof(*pipe_fds));
 	if (!pipe_fds) {
 		pr_inf_skip("%s: out of memory allocating %zd pipe file descriptors, "
 			"skipping stressor\n", args->name, max_fds);
 		return EXIT_NO_RESOURCE;
 	}
-	poll_fds = calloc((size_t)max_fds, sizeof(*poll_fds));
+	poll_fds = (struct pollfd *)calloc(max_fds, sizeof(*poll_fds));
 	if (!poll_fds) {
 		pr_inf("%s: out of memory allocating %zd poll file descriptors, "
 			"skipping stressor\n", args->name, max_fds);
@@ -122,6 +126,31 @@ static int stress_poll(const stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
+	max_rnd_fds = max_fds;
+	if (max_rnd_fds < MAX_POLL_FDS)
+		max_rnd_fds = MAX_POLL_FDS - (MAX_POLL_FDS % max_fds);
+
+	rnd_fds_index = (int *)calloc(max_rnd_fds, sizeof(*rnd_fds_index));
+	if (!rnd_fds_index) {
+		pr_inf("%s: out of memory allocating %zd randomized poll indices, "
+			"skipping stressor\n", args->name, max_fds);
+		free(poll_fds);
+		free(pipe_fds);
+		return EXIT_NO_RESOURCE;
+	}
+
+	/* randomize: shuffle rnd_fds_index */
+	for (i = 0; i < max_rnd_fds; i++) {
+		rnd_fds_index[i] = i % max_fds;
+	}
+	for (i = 0; i < max_rnd_fds; i++) {
+		register size_t tmp, j = stress_mwc32modn(max_rnd_fds);
+
+		tmp = rnd_fds_index[i];
+		rnd_fds_index[i] = rnd_fds_index[j];
+		rnd_fds_index[j] = tmp;
+	}
+
 	for (i = 0; i < max_fds; i++) {
 		if (pipe(pipe_fds[i].fd) < 0) {
 			size_t j;
@@ -130,22 +159,26 @@ static int stress_poll(const stress_args_t *args)
 				args->name, errno, strerror(errno));
 
 			for (j = 0; j < i; j++) {
-				(void)close(pipe_fds[i].fd[0]);
-				(void)close(pipe_fds[i].fd[1]);
+				(void)close(pipe_fds[j].fd[0]);
+				(void)close(pipe_fds[j].fd[1]);
 			}
+			free(rnd_fds_index);
 			free(poll_fds);
 			free(pipe_fds);
 			return EXIT_NO_RESOURCE;
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto tidy;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -154,28 +187,33 @@ again:
 	} else if (pid == 0) {
 		/* Child writer */
 
+		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
 		for (i = 0; i < max_fds; i++)
 			(void)close(pipe_fds[i].fd[0]);
 
+		i = 0;
 		do {
-			char buf[POLL_BUF];
-			ssize_t ret;
+			register size_t j = rnd_fds_index[i];
+			uint16_t buf ALIGN64 = (uint16_t)j;
+			register ssize_t ret;
+
+			i++;
+			if (UNLIKELY(i >= max_rnd_fds))
+				i = 0;
 
 			/* Write on a randomly chosen pipe */
-			i = (stress_mwc32() >> 8) % max_fds;
-			(void)memset(buf, (int)('0' + i), sizeof(buf));
-			ret = write(pipe_fds[i].fd[1], buf, sizeof(buf));
-			if (ret < (ssize_t)sizeof(buf)) {
+			ret = write(pipe_fds[j].fd[1], &buf, sizeof(buf));
+			if (UNLIKELY(ret < (ssize_t)sizeof(buf))) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					continue;
 				pr_fail("%s: write failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto abort;
 			}
-		 } while (keep_stressing(args));
+		 } while (stress_continue(args));
 abort:
 		for (i = 0; i < max_fds; i++)
 			(void)close(pipe_fds[i].fd[1]);
@@ -183,7 +221,6 @@ abort:
 	} else {
 		/* Parent read */
 
-		int status;
 #if defined(HAVE_SYS_SELECT_H) &&	\
     defined(HAVE_SELECT)
 		int maxfd = 0;
@@ -201,6 +238,15 @@ abort:
 			poll_fds[i].revents = 0;
 		}
 
+		/*
+		 * Exercise POLLNVAL revents being set on fds[1] by
+		 * setting the fd to a highly probable invalid fd
+		 * number. This should not make poll fail and should
+		 * set poll_fds[1].revents to POLLNVAL
+		 */
+		if (max_fds > 2)
+			poll_fds[1].fd = 10000;
+
 		do {
 #if defined(HAVE_SYS_SELECT_H) &&	\
     defined(HAVE_SELECT)
@@ -208,61 +254,61 @@ abort:
 #endif
 			int ret;
 
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			/* stress out poll */
 			ret = poll(poll_fds, max_fds, 1);
-			if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-			    (ret < 0) && (errno != EINTR)) {
+			if (UNLIKELY((g_opt_flags & OPT_FLAGS_VERIFY) &&
+				     (ret < 0) && (errno != EINTR))) {
 				pr_fail("%s: poll failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
-			if (ret > 0) {
+			if (LIKELY(ret > 0)) {
 				for (i = 0; i < max_fds; i++) {
 					if (poll_fds[i].revents == POLLIN) {
-						if (pipe_read(args, poll_fds[i].fd, i) < 0)
+						if (UNLIKELY(pipe_read(args, poll_fds[i].fd, i) < 0))
 							break;
 					}
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
 
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 #if defined(HAVE_PPOLL)
 			/* stress out ppoll */
 
 			ts.tv_sec = 0;
-			ts.tv_nsec = 20000000;
+			ts.tv_nsec = poll_random_us ? (long)stress_mwc32modn(poll_random_us * 1000) : 20000000L;
 
 			(void)sigemptyset(&sigmask);
 			(void)sigaddset(&sigmask, SIGPIPE);
 
-			ret = ppoll(poll_fds, max_fds, &ts, &sigmask);
-			if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-			    (ret < 0) && (errno != EINTR)) {
+			ret = shim_ppoll(poll_fds, max_fds, &ts, &sigmask);
+			if (UNLIKELY((g_opt_flags & OPT_FLAGS_VERIFY) &&
+				     (ret < 0) && (errno != EINTR))) {
 				pr_fail("%s: ppoll failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
-			if (ret > 0) {
+			if (LIKELY(ret > 0)) {
 				for (i = 0; i < max_fds; i++) {
 					if (poll_fds[i].revents == POLLIN) {
-						if (pipe_read(args, poll_fds[i].fd, i) < 0)
+						if (UNLIKELY(pipe_read(args, poll_fds[i].fd, i) < 0))
 							break;
 					}
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			/* Exercise illegal poll timeout */
 			ts.tv_sec = 0;
 			ts.tv_nsec = 1999999999;
-			VOID_RET(int, ppoll(poll_fds, max_fds, &ts, &sigmask));
-			if (!keep_stressing(args))
+			VOID_RET(int, shim_ppoll(poll_fds, max_fds, &ts, &sigmask));
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 #if defined(RLIMIT_NOFILE)
@@ -273,17 +319,17 @@ abort:
 			{
 				struct rlimit old_rlim, new_rlim;
 
-				if (getrlimit(RLIMIT_NOFILE, &old_rlim) == 0) {
+				if (LIKELY(getrlimit(RLIMIT_NOFILE, &old_rlim) == 0)) {
 					new_rlim.rlim_cur = max_fds - 1;
 					new_rlim.rlim_max = old_rlim.rlim_max;
 
-					if (setrlimit(RLIMIT_NOFILE, &new_rlim) == 0) {
+					if (LIKELY(setrlimit(RLIMIT_NOFILE, &new_rlim) == 0)) {
 						ts.tv_sec = 0;
 						ts.tv_nsec = 0;
-						VOID_RET(int, ppoll(poll_fds, max_fds, &ts, &sigmask));
+						VOID_RET(int, shim_ppoll(poll_fds, max_fds, &ts, &sigmask));
 
 						(void)setrlimit(RLIMIT_NOFILE, &old_rlim);
-						if (!keep_stressing(args))
+						if (UNLIKELY(!stress_continue(args)))
 							break;
 					}
 				}
@@ -299,36 +345,36 @@ abort:
 			for (i = 0; i < max_fds; i++) {
 				if (pipe_fds[i].fd[0] < FD_SETSIZE) {
 					FD_SET(pipe_fds[i].fd[0], &rfds);
-					if (pipe_fds[i].fd[0] > maxfd)
+					if (LIKELY(pipe_fds[i].fd[0] > maxfd))
 						maxfd = pipe_fds[i].fd[0];
 				}
 			}
 			tv.tv_sec = 0;
-			tv.tv_usec = 20000;
+			tv.tv_usec = poll_random_us ? (long)stress_mwc32modn(poll_random_us) : 20000L;
 			ret = select(maxfd + 1, &rfds, NULL, NULL, &tv);
-			if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-			    (ret < 0) && (errno != EINTR)) {
+			if (UNLIKELY((g_opt_flags & OPT_FLAGS_VERIFY) &&
+				     (ret < 0) && (errno != EINTR))) {
 				pr_fail("%s: select failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
-			if (ret > 0) {
+			if (LIKELY(ret > 0)) {
 				for (i = 0; i < max_fds; i++) {
 					if ((pipe_fds[i].fd[0] < FD_SETSIZE) &&
 					     FD_ISSET(pipe_fds[i].fd[0], &rfds)) {
-						if (pipe_read(args, pipe_fds[i].fd[0], i) < 0)
+						if (UNLIKELY(pipe_read(args, pipe_fds[i].fd[0], i) < 0))
 							break;
 					}
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 #endif
 
 #if defined(HAVE_PSELECT)
 			/* stress out pselect */
 			ts.tv_sec = 0;
-			ts.tv_nsec = 20000000;
+			ts.tv_nsec = poll_random_us ? (long)stress_mwc32modn(poll_random_us * 1000) : 20000000L;
 
 			(void)sigemptyset(&sigmask);
 			(void)sigaddset(&sigmask, SIGPIPE);
@@ -337,25 +383,25 @@ abort:
 			for (i = 0; i < max_fds; i++) {
 				if (pipe_fds[i].fd[0] < FD_SETSIZE) {
 					FD_SET(pipe_fds[i].fd[0], &rfds);
-					if (pipe_fds[i].fd[0] > maxfd)
+					if (LIKELY(pipe_fds[i].fd[0] > maxfd))
 						maxfd = pipe_fds[i].fd[0];
 				}
 			}
 			ret = pselect(maxfd + 1, &rfds, NULL, NULL, &ts, &sigmask);
-			if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-			    (ret < 0) && (errno != EINTR)) {
+			if (UNLIKELY((g_opt_flags & OPT_FLAGS_VERIFY) &&
+				     (ret < 0) && (errno != EINTR))) {
 				pr_fail("%s: pselect failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
-			if (ret > 0) {
+			if (LIKELY(ret > 0)) {
 				for (i = 0; i < max_fds; i++) {
 					if ((pipe_fds[i].fd[0] < FD_SETSIZE) &&
 					    (FD_ISSET(pipe_fds[i].fd[0], &rfds))) {
-						if (pipe_read(args, pipe_fds[i].fd[0], i) < 0)
+						if (UNLIKELY(pipe_read(args, pipe_fds[i].fd[0], i) < 0))
 							break;
 					}
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
 #endif
 			/*
@@ -363,10 +409,9 @@ abort:
 			 * a select zero timeout
 			 */
 			(void)sleep(0);
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
-		(void)kill(pid, SIGKILL);
-		(void)shim_waitpid(pid, &status, 0);
+		(void)stress_kill_pid_wait(pid, NULL);
 	}
 
 tidy:
@@ -377,6 +422,7 @@ tidy:
 		(void)close(pipe_fds[i].fd[1]);
 	}
 
+	free(rnd_fds_index);
 	free(poll_fds);
 	free(pipe_fds);
 
@@ -384,18 +430,19 @@ tidy:
 }
 
 
-stressor_info_t stress_poll_info = {
+const stressor_info_t stress_poll_info = {
 	.stressor = stress_poll,
 	.class = CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
-stressor_info_t stress_poll_info = {
+const stressor_info_t stress_poll_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without poll.h"
 };
