@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-pragma.h"
 
 #define MIN_REVIO_BYTES		(1 * MB)
 #define MAX_REVIO_BYTES		(MAX_FILE_LIMIT)
@@ -49,9 +50,11 @@
 #define REVIO_OPT_SYNCFS	(0x00800000)
 
 static const stress_help_t help[] = {
-	{ NULL,	"revio N",	"start N workers performing reverse I/O" },
-	{ NULL,	"revio-ops N",	"stop after N revio bogo operations" },
-	{ NULL,	NULL,		NULL }
+	{ NULL,	"revio N",		"start N workers performing reverse I/O" },
+	{ NULL, "revio-bytes N",	"specify file size (default is 1GB)" },
+	{ NULL,	"revio-ops N",		"stop after N revio bogo operations" },
+	{ NULL, "revio-opts list",	"specify list of various stressor options" },
+	{ NULL,	NULL,			NULL }
 };
 
 typedef struct {
@@ -125,16 +128,6 @@ static const stress_revio_opts_t revio_opts[] = {
 	{ "utimes",	REVIO_OPT_UTIMES, 0, 0, 0 },
 };
 
-static int stress_set_revio_bytes(const char *opt)
-{
-	uint64_t revio_bytes;
-
-	revio_bytes = stress_get_uint64_byte_filesystem(opt, 1);
-	stress_check_range_bytes("revio-bytes", revio_bytes,
-		MIN_REVIO_BYTES, MAX_REVIO_BYTES);
-	return stress_set_setting("revio-bytes", TYPE_ID_UINT64, &revio_bytes);
-}
-
 /*
  *  stress_revio_write()
  *	write with writev or write depending on mode
@@ -155,25 +148,25 @@ static ssize_t stress_revio_write(
 #endif
 
 	ret = write(fd, buf, count);
-	if (!keep_stressing_flag())
+	if (UNLIKELY(!stress_continue_flag()))
 		return ret;
 
 #if defined(HAVE_FSYNC)
 	if (revio_flags & REVIO_OPT_FSYNC)
 		(void)shim_fsync(fd);
-	if (!keep_stressing_flag())
+	if (UNLIKELY(!stress_continue_flag()))
 		return ret;
 #endif
 #if defined(HAVE_FDATASYNC)
 	if (revio_flags & REVIO_OPT_FDATASYNC)
 		(void)shim_fdatasync(fd);
-	if (!keep_stressing_flag())
+	if (UNLIKELY(!stress_continue_flag()))
 		return ret;
 #endif
 #if defined(HAVE_SYNCFS)
 	if (revio_flags & REVIO_OPT_SYNCFS)
 		(void)syncfs(fd);
-	if (!keep_stressing_flag())
+	if (UNLIKELY(!stress_continue_flag()))
 		return ret;
 #endif
 
@@ -181,19 +174,25 @@ static ssize_t stress_revio_write(
 }
 
 /*
- *  stress_set_revio_opts
+ *  stress_revio_opts
  *	parse --revio-opts option(s) list
  */
-static int stress_set_revio_opts(const char *opts)
+static void stress_revio_opts(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
 {
-	char *str, *ptr, *token;
+	char *str, *ptr;
+	const char *token;
 	int revio_flags = 0;
 	int revio_oflags = 0;
 	bool opts_set = false;
 
-	str = stress_const_optdup(opts);
-	if (!str)
-		return -1;
+	(void)type_id;
+	(void)value;
+
+	str = stress_const_optdup(opt_arg);
+	if (!str) {
+		(void)fprintf(stderr, "%s option: cannot dup string '%s'\n", opt_name, opt_arg);
+		longjmp(g_error_env, 1);
+	}
 
 	for (ptr = str; (token = strtok(ptr, ",")) != NULL; ptr = NULL) {
 		size_t i;
@@ -201,22 +200,24 @@ static int stress_set_revio_opts(const char *opts)
 
 		for (i = 0; i < SIZEOF_ARRAY(revio_opts); i++) {
 			if (!strcmp(token, revio_opts[i].opt)) {
-				int exclude = revio_flags & revio_opts[i].exclude;
+				const int exclude = revio_flags & revio_opts[i].exclude;
+
 				if (exclude) {
 					int j;
 
 					for (j = 0; revio_opts[j].opt; j++) {
 						if ((exclude & revio_opts[j].flag) == exclude) {
 							(void)fprintf(stderr,
-								"revio-opt option '%s' is not "
+								"%s option '%s' is not "
 								"compatible with option '%s'\n",
-								token,
+								opt_name, token,
 								revio_opts[j].opt);
-							break;
+							free(str);
+							longjmp(g_error_env, 1);
 						}
 					}
 					free(str);
-					return -1;
+					return;
 				}
 				revio_flags  |= revio_opts[i].flag;
 				revio_oflags |= revio_opts[i].oflag;
@@ -225,29 +226,26 @@ static int stress_set_revio_opts(const char *opts)
 			}
 		}
 		if (!opt_ok) {
-			(void)fprintf(stderr, "revio-opt option '%s' not known, options are:", token);
+			(void)fprintf(stderr, "%s option '%s' not known, options are:", opt_name, token);
 			for (i = 0; i < SIZEOF_ARRAY(revio_opts); i++)
-				(void)fprintf(stderr, "%s %s",
-					i == 0 ? "" : ",", revio_opts[i].opt);
+				(void)fprintf(stderr, " %s", revio_opts[i].opt);
 			(void)fprintf(stderr, "\n");
 			free(str);
-			return -1;
+			longjmp(g_error_env, 1);
 		}
 	}
 
-	stress_set_setting("revio-flags", TYPE_ID_INT, &revio_flags);
-	stress_set_setting("revio-oflags", TYPE_ID_INT, &revio_oflags);
-	stress_set_setting("revio-opts-set", TYPE_ID_BOOL, &opts_set);
+	stress_set_setting("revio", "revio-flags", TYPE_ID_INT, &revio_flags);
+	stress_set_setting("revio", "revio-oflags", TYPE_ID_INT, &revio_oflags);
+	stress_set_setting("revio", "revio-opts-set", TYPE_ID_BOOL, &opts_set);
 	free(str);
-
-	return 0;
 }
 
 /*
  *  stress_revio_advise()
  *	set posix_fadvise options
  */
-static int stress_revio_advise(const stress_args_t *args, const int fd, const int flags)
+static int stress_revio_advise(stress_args_t *args, const int fd, const int flags)
 {
 #if (defined(POSIX_FADV_SEQ) || defined(POSIX_FADV_RANDOM) ||		\
      defined(POSIX_FADV_NOREUSE) || defined(POSIX_FADV_WILLNEED) ||	\
@@ -258,7 +256,7 @@ static int stress_revio_advise(const stress_args_t *args, const int fd, const in
 	if (!(flags & REVIO_OPT_FADV_MASK))
 		return 0;
 
-	for (i = 0; keep_stressing(args) && (i < SIZEOF_ARRAY(revio_opts)); i++) {
+	for (i = 0; LIKELY(stress_continue(args) && (i < SIZEOF_ARRAY(revio_opts))); i++) {
 		if (revio_opts[i].flag & flags) {
 			if (posix_fadvise(fd, 0, 0, revio_opts[i].advice) < 0) {
 				pr_fail("%s: posix_fadvise failed, errno=%d (%s)\n",
@@ -279,7 +277,7 @@ static int stress_revio_advise(const stress_args_t *args, const int fd, const in
  *  stress_revio
  *	stress I/O via writes in reverse
  */
-static int stress_revio(const stress_args_t *args)
+static int stress_revio(stress_args_t *args)
 {
 	uint8_t *buf = NULL;
 	void *alloc_buf;
@@ -311,7 +309,7 @@ static int stress_revio(const stress_args_t *args)
 			revio_bytes = MIN_REVIO_BYTES;
 	}
 
-	revio_bytes /= args->num_instances;
+	revio_bytes /= args->instances;
 
 	/* Ensure complete file size is not less than the I/O size */
 	if (revio_bytes < DEFAULT_REVIO_WRITE_SIZE) {
@@ -350,6 +348,8 @@ static int stress_revio(const stress_args_t *args)
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -362,53 +362,63 @@ static int stress_revio(const stress_args_t *args)
 		 * the "work through all the options" mode
 		 */
 		if (!opts_set && (g_opt_flags & OPT_FLAGS_AGGRESSIVE)) {
-			opt_index = (opt_index + 1) % SIZEOF_ARRAY(revio_opts);
+			opt_index++;
+			if (opt_index >= SIZEOF_ARRAY(revio_opts))
+				opt_index = 0;
 
 			revio_flags = revio_opts[opt_index].flag;
 			revio_oflags = revio_opts[opt_index].oflag;
 		}
 
-		if ((fd = open(filename, flags, S_IRUSR | S_IWUSR)) < 0) {
+		if (UNLIKELY((fd = open(filename, flags, S_IRUSR | S_IWUSR)) < 0)) {
 			if ((errno == ENOSPC) || (errno == ENOMEM))
 				continue;	/* Retry */
 			pr_fail("%s: open %s failed, errno=%d (%s)\n",
 				args->name, filename, errno, strerror(errno));
+			rc = EXIT_FAILURE;
 			goto finish;
 		}
-		fs_type = stress_fs_type(filename);
+		stress_file_rw_hint_short(fd);
+
+		fs_type = stress_get_fs_type(filename);
 		(void)shim_unlink(filename);
-		if (ftruncate(fd, (off_t)revio_bytes) < 0) {
+		if (UNLIKELY(ftruncate(fd, (off_t)revio_bytes) < 0)) {
 			pr_fail("%s: ftruncate failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			(void)close(fd);
+			rc = EXIT_FAILURE;
 			goto finish;
 		}
 
-		if (stress_revio_advise(args, fd, fadvise_flags) < 0) {
+		if (UNLIKELY(stress_revio_advise(args, fd, fadvise_flags) < 0)) {
 			(void)close(fd);
+			rc = EXIT_FAILURE;
 			goto finish;
 		}
 
 		/* Sequential Reverse Write */
 		for (i = 0; i < revio_bytes; i += DEFAULT_REVIO_WRITE_SIZE * (8 + (stress_mwc8() & 7))) {
 			size_t j;
-			off_t lseek_ret, offset = (off_t)(revio_bytes - i);
+			off_t lseek_ret;
+			const off_t offset = (off_t)(revio_bytes - i);
 seq_wr_retry:
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			lseek_ret = lseek(fd, offset, SEEK_SET);
-			if (lseek_ret < 0) {
+			if (UNLIKELY(lseek_ret < 0)) {
 				pr_fail("%s: write failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
 				(void)close(fd);
+				rc = EXIT_FAILURE;
 				goto finish;
 			}
 
+PRAGMA_UNROLL_N(4)
 			for (j = 0; j < DEFAULT_REVIO_WRITE_SIZE; j += 512)
 				buf[j] = (i * j) & 0xff;
 			ret = stress_revio_write(fd, buf, (size_t)DEFAULT_REVIO_WRITE_SIZE, revio_flags);
-			if (ret <= 0) {
+			if (UNLIKELY(ret <= 0)) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					goto seq_wr_retry;
 				if (errno == ENOSPC)
@@ -417,21 +427,23 @@ seq_wr_retry:
 					pr_fail("%s: write failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno), fs_type);
 					(void)close(fd);
+					rc = EXIT_FAILURE;
 					goto finish;
 				}
 				continue;
 			}
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 		iterations++;
 		extents = stress_get_extents(fd);
 		avg_extents += (double)extents;
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	if ((iterations > 0) && (avg_extents > 0.0)) {
 		avg_extents /= (double)iterations;
-		pr_inf("%s: average number of extents %.2f\n", args->name, avg_extents * args->num_instances);
+		stress_metrics_set(args, 0, "extents",
+			(double)avg_extents, STRESS_METRIC_GEOMETRIC_MEAN);
 	}
 
 	rc = EXIT_SUCCESS;
@@ -443,17 +455,17 @@ finish:
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_revio_bytes,	stress_set_revio_bytes },
-	{ OPT_revio_opts,	stress_set_revio_opts },
-	{ 0,			NULL }
 
+static const stress_opt_t opts[] = {
+	{ OPT_revio_bytes, "revio-bytes", TYPE_ID_UINT64_BYTES_VM, MIN_REVIO_BYTES, MAX_REVIO_BYTES, NULL },
+	{ OPT_revio_opts,  "revio-opts",  TYPE_ID_CALLBACK, 0, 0, stress_revio_opts },
+	END_OPT,
 };
 
-stressor_info_t stress_revio_info = {
+const stressor_info_t stress_revio_info = {
 	.stressor = stress_revio,
 	.class = CLASS_IO | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
