@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_LINUX_AUDIT_H)
 #include <linux/audit.h>
@@ -117,22 +118,22 @@ static struct sock_filter filter[] = {
 static struct sock_filter filter_random[64];
 
 static struct sock_fprog prog_allow_all = {
-	.len = (unsigned short)SIZEOF_ARRAY(filter_allow_all),
+	.len = (unsigned short int)SIZEOF_ARRAY(filter_allow_all),
 	.filter = filter_allow_all
 };
 
 static struct sock_fprog prog_allow_write = {
-	.len = (unsigned short)SIZEOF_ARRAY(filter_allow_write),
+	.len = (unsigned short int)SIZEOF_ARRAY(filter_allow_write),
 	.filter = filter_allow_write
 };
 
 static struct sock_fprog prog = {
-	.len = (unsigned short)SIZEOF_ARRAY(filter),
+	.len = (unsigned short int)SIZEOF_ARRAY(filter),
 	.filter = filter
 };
 
 static struct sock_fprog prog_random = {
-	.len = (unsigned short)SIZEOF_ARRAY(filter_random),
+	.len = (unsigned short int)SIZEOF_ARRAY(filter_random),
 	.filter = filter_random
 };
 
@@ -217,7 +218,7 @@ static void MLOCKED_TEXT NORETURN stress_sigsys(int signum)
  *  stress_seccomp_set_huge_filter()
  *	set up a huge seccomp filter, see how large we can go
  */
-static int stress_seccomp_set_huge_filter(const stress_args_t *args)
+static int stress_seccomp_set_huge_filter(stress_args_t *args)
 {
 	static struct sock_filter bpf_stmt =
 		BPF_STMT(BPF_RET + BPF_K, SECCOMP_RET_ALLOW);
@@ -226,7 +227,7 @@ static int stress_seccomp_set_huge_filter(const stress_args_t *args)
 	const size_t n_max = ((size_t)1 << bits) - 1;
 	size_t i, j, n = 32, max = 1;
 
-	(void)memset(&huge_prog, 0, sizeof(huge_prog));
+	(void)shim_memset(&huge_prog, 0, sizeof(huge_prog));
 	if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0) < 0) {
 		pr_fail("%s: prctl PR_SET_NEW_PRIVS failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -236,13 +237,13 @@ static int stress_seccomp_set_huge_filter(const stress_args_t *args)
 	for (j = 0; (n < n_max) && (n != max) && (j < 64); j++) {
 		struct sock_filter *huge_filter;
 
-		huge_filter = calloc(n, sizeof(*huge_filter));
+		huge_filter = (struct sock_filter *)calloc(n, sizeof(*huge_filter));
 		if (!huge_filter)
 			return -1;
 
 		for (i = 0; i < n; i++)
 			huge_filter[i] = bpf_stmt;
-		huge_prog.len = (unsigned short)n;
+		huge_prog.len = (unsigned short int)n;
 		huge_prog.filter = huge_filter;
 
 #if defined(__NR_seccomp)
@@ -275,7 +276,7 @@ next:
  * 	if allow_write is true.
  */
 static int stress_seccomp_set_filter(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const bool allow_write,
 	bool do_random)
 {
@@ -380,14 +381,16 @@ redo_prctl:
  *  stress_seccomp()
  *	stress seccomp
  */
-static int stress_seccomp(const stress_args_t *args)
+static int stress_seccomp(stress_args_t *args)
 {
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		pid_t pid;
-		const bool allow_write = (stress_mwc32() % 2) != 0;
-		const bool do_random = (stress_mwc32() % 20) != 0;
+		const bool allow_write = stress_mwc1() != 0;
+		const bool do_random = stress_mwc32modn(20) != 0;
 
 		pid = fork();
 		if (pid == -1) {
@@ -431,7 +434,7 @@ static int stress_seccomp(const stress_args_t *args)
 			/* Wait for child to exit or get killed by seccomp */
 			if (shim_waitpid(pid, &status, 0) < 0) {
 				if (errno != EINTR)
-					pr_dbg("%s: waitpid failed, errno = %d (%s)\n",
+					pr_dbg("%s: waitpid failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 			} else {
 				/* Did the child hit a weird error? */
@@ -449,6 +452,7 @@ static int stress_seccomp(const stress_args_t *args)
 						"but got a successful exit which "
 						"was not expected\n",
 						args->name);
+					return EXIT_FAILURE;
 				}
 				/* ..exited with a SIGSYS but we expected OK exit? */
 				if (WIFSIGNALED(status) && allow_write) {
@@ -457,19 +461,20 @@ static int stress_seccomp(const stress_args_t *args)
 							"but got a seccomp SIGSYS "
 							"which was not expected\n",
 							args->name);
+						return EXIT_FAILURE;
 					}
 				}
 			}
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_seccomp_info = {
+const stressor_info_t stress_seccomp_info = {
 	.stressor = stress_seccomp,
 	.supported = stress_seccomp_supported,
 	.class = CLASS_OS,
@@ -477,9 +482,10 @@ stressor_info_t stress_seccomp_info = {
 	.help = help
 };
 #else
-stressor_info_t stress_seccomp_info = {
+const stressor_info_t stress_seccomp_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/seccomp.h, linux/audit.h, linux/filter.h or prctl()"
 };
