@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include <sys/socket.h>
 
 #if defined(HAVE_LINUX_NETLINK_H)
 #include <linux/netlink.h>
@@ -60,7 +61,7 @@ typedef struct {
 	struct unix_diag_req udr;
 } stress_sockdiag_request_t;
 
-static int sockdiag_send(const stress_args_t *args, const int fd)
+static int sockdiag_send(stress_args_t *args, const int fd)
 {
 	static size_t family = 0;
 
@@ -171,7 +172,7 @@ static int sockdiag_send(const stress_args_t *args, const int fd)
 #endif
 	};
 
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		ssize_t ret;
 		unsigned int i;
 
@@ -180,20 +181,20 @@ static int sockdiag_send(const stress_args_t *args, const int fd)
 		for (i = 0; i < 32; i++) {
 			request.udr.udiag_show = 1U << i;
 			ret = sendmsg(fd, &msg, 0);
-			if (ret > 0)
+			if (LIKELY(ret > 0))
 				return 1;
-			if (errno != EINTR)
+			if (LIKELY(errno != EINTR))
 				return -1;
 		}
 		request.udr.udiag_show = ~0U;
 		ret = sendmsg(fd, &msg, 0);
-		if (ret > 0)
+		if (LIKELY(ret > 0))
 			return 1;
-		if (errno != EINTR)
+		if (LIKELY(errno != EINTR))
 			return -1;
 
 		family++;
-		if (family >= SIZEOF_ARRAY(families))
+		if (UNLIKELY(family >= SIZEOF_ARRAY(families)))
 			family = 0;
 	}
 
@@ -201,31 +202,32 @@ static int sockdiag_send(const stress_args_t *args, const int fd)
 }
 
 static int stress_sockdiag_parse(
-	const stress_args_t *args,
+	stress_args_t *args,
 	struct unix_diag_msg *diag,
 	unsigned int len)
 {
 	struct rtattr *attr;
 	unsigned int rta_len;
+	uint32_t count = 0;
 
-	if (len < NLMSG_LENGTH(sizeof(*diag))) {
+	if (UNLIKELY(len < NLMSG_LENGTH(sizeof(*diag)))) {
 		/* short response, ignore for now */
 		return 0;
 	}
 
 	rta_len = len - NLMSG_LENGTH(sizeof(*diag));
-	for (attr = (struct rtattr *) (diag + 1);
-	     RTA_OK(attr, rta_len) && keep_stressing(args);
+	for (attr = (struct rtattr *) (diag + 1); RTA_OK(attr, rta_len);
 	     attr = RTA_NEXT(attr, rta_len)) {
-		inc_counter(args);
+		count++;
 	}
+	stress_bogo_add(args, (uint64_t)count);
 
 	return 0;
 }
 
-static int sockdiag_recv(const stress_args_t *args, const int fd)
+static int sockdiag_recv(stress_args_t *args, const int fd)
 {
-	static uint32_t buf[4096];
+	static uint32_t buf[4096] ALIGN64;
 	int flags = 0;
 
 	static struct sockaddr_nl nladdr = {
@@ -239,34 +241,33 @@ static int sockdiag_recv(const stress_args_t *args, const int fd)
 
 	for (;;) {
 		ssize_t ret;
-		static struct msghdr msg = {
+		static struct msghdr msg ALIGN64 = {
 			.msg_name = (void *)&nladdr,
 			.msg_namelen = sizeof(nladdr),
 			.msg_iov = &iov,
 			.msg_iovlen = 1
-		};
+		} ;
 		struct nlmsghdr *h = (struct nlmsghdr *)buf;
 
 		ret = recvmsg(fd, &msg, flags);
-		if (ret == 0)
-			break;
-		if (ret < 0) {
+		if (UNLIKELY(ret <= 0)) {
+			if (ret == 0)
+				break;
 			if (errno == EINTR)
 				continue;
-
 			return -1;
 		}
-		if (!NLMSG_OK(h, ret))
+		if (UNLIKELY(!NLMSG_OK(h, ret)))
 			return -1;
 
 		for (; NLMSG_OK(h, ret); h = NLMSG_NEXT(h, ret)) {
 			if (h->nlmsg_type == NLMSG_DONE)
 				return 0;
 
-			if (h->nlmsg_type == NLMSG_ERROR)
+			if (UNLIKELY(h->nlmsg_type == NLMSG_ERROR))
 				return -1;
 
-			if (h->nlmsg_type != SOCK_DIAG_BY_FAMILY)
+			if (UNLIKELY(h->nlmsg_type != SOCK_DIAG_BY_FAMILY))
 				return -1;
 
 			if (stress_sockdiag_parse(args, NLMSG_DATA(h), h->nlmsg_len))
@@ -280,59 +281,64 @@ static int sockdiag_recv(const stress_args_t *args, const int fd)
  *  stress_sockdiag
  *	stress by heavy socket I/O
  */
-static int stress_sockdiag(const stress_args_t *args)
+static int stress_sockdiag(stress_args_t *args)
 {
-	int ret = EXIT_SUCCESS;
+	int rc = EXIT_SUCCESS;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		int fd, rc;
+		int fd, ret;
 
 		fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_SOCK_DIAG);
-		if (fd < 0) {
+		if (UNLIKELY(fd < 0)) {
 			if (errno == EPROTONOSUPPORT) {
 				if (args->instance == 0)
-					pr_inf("%s: NETLINK_SOCK_DIAG not supported, skipping stressor\n",
+					pr_inf_skip("%s: NETLINK_SOCK_DIAG not supported, skipping stressor\n",
 						args->name);
-				ret = EXIT_NOT_IMPLEMENTED;
+				rc  = EXIT_NOT_IMPLEMENTED;
 				break;
 			}
-			pr_err("%s: NETLINK_SOCK_DIAG open failed: errno=%d (%s)\n",
+			pr_fail("%s: NETLINK_SOCK_DIAG open failed: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			ret = EXIT_FAILURE;
+			rc = EXIT_FAILURE;
 			break;
 		}
-		rc = sockdiag_send(args, fd);
-		if (rc < 0) {
+		ret = sockdiag_send(args, fd);
+		if (UNLIKELY(ret < 0)) {
 			pr_fail("%s: NETLINK_SOCK_DIAG send query failed: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			ret = EXIT_FAILURE;
+			rc = EXIT_FAILURE;
 			(void)close(fd);
 			break;
-		} else if (rc == 0) {
+		} else if (ret == 0) {
 			/* Nothing sent or timed out? */
 			(void)close(fd);
 			break;
 		}
 		(void)sockdiag_recv(args, fd);
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	return ret;
+	return rc;
 }
 
-stressor_info_t stress_sockdiag_info = {
+const stressor_info_t stress_sockdiag_info = {
 	.stressor = stress_sockdiag,
 	.class = CLASS_NETWORK | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_sockdiag_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_sockdiag_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK | CLASS_OS,
-	.help = help
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without linux/sock_diag.h, linux/netlink.h, linux/rtnetlink.h or linux/unix_diag.h"
 };
 #endif
