@@ -208,7 +208,7 @@ static int OPTIMIZE3 stress_vm_parent(stress_context_t *ctxt)
 
 		/* Wait for address of child's buffer */
 redo_rd2:
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		rwret = read(ctxt->pipe_wr[0], &msg_rd, sizeof(msg_rd));
 		if (UNLIKELY(rwret < 0)) {
@@ -305,7 +305,7 @@ redo_rd2:
 		msg_wr.val = val;
 		val++;
 redo_wr2:
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		/* Inform child that memory has been changed */
 		rwret = write(ctxt->pipe_rd[1], &msg_wr, sizeof(msg_wr));
@@ -373,7 +373,7 @@ static int stress_vm_rw(stress_args_t *args)
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			vm_rw_bytes = MIN_VM_RW_BYTES;
 	}
-	vm_rw_bytes /= args->num_instances;
+	vm_rw_bytes /= args->instances;
 	if (vm_rw_bytes < MIN_VM_RW_BYTES)
 		vm_rw_bytes = MIN_VM_RW_BYTES;
 	if (vm_rw_bytes < args->page_size)
