@@ -98,8 +98,11 @@ static const stress_vm_madvise_info_t vm_madvise_info[] = {
 	/* No MADVISE, default to normal, ignored */
 	{ "normal",	0 },
 #else
+#if defined(MADV_COLLAPSE)
+	{ "collapse",	MADV_COLLAPSE },
+#endif
 #if defined(MADV_DONTNEED)
-	{ "dontneed",	MADV_DONTNEED},
+	{ "dontneed",	MADV_DONTNEED },
 #endif
 #if defined(MADV_HUGEPAGE)
 	{ "hugepage",	MADV_HUGEPAGE },
@@ -328,6 +331,8 @@ static size_t TARGET_CLONES stress_vm_moving_inversion(
 	if (UNLIKELY(!stress_continue_flag()))
 		goto ret;
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 
 	inject_random_bit_errors(buf, sz);
@@ -354,6 +359,8 @@ static size_t TARGET_CLONES stress_vm_moving_inversion(
 
 	inject_random_bit_errors(buf, sz);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	stress_mwc_set_seed(w, z);
 	for (ptr = (uint64_t *)buf_end; ptr > (uint64_t *)buf; ) {
@@ -451,6 +458,8 @@ static size_t TARGET_CLONES stress_vm_modulo_x(
 
 	if (UNLIKELY(max_ops && (c >= max_ops)))
 		c = max_ops;
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("modulo X", bit_errors);
 ret:
 	stress_bogo_set(args, c);
@@ -498,6 +507,8 @@ static size_t TARGET_CLONES stress_vm_walking_one_data(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("walking one (data)", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -544,6 +555,8 @@ static size_t TARGET_CLONES stress_vm_walking_zero_data(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("walking zero (data)", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -591,6 +604,8 @@ static size_t TARGET_CLONES stress_vm_walking_one_addr(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("walking one (address)", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -644,6 +659,8 @@ static size_t TARGET_CLONES stress_vm_walking_zero_addr(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("walking zero (address)", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -716,6 +733,8 @@ static size_t TARGET_CLONES stress_vm_walking_flush_data(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("walking flush (data)", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -801,6 +820,8 @@ static size_t TARGET_CLONES OPTIMIZE3 stress_vm_gray(
 	}
 	val++;
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("gray code", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -875,6 +896,8 @@ static size_t TARGET_CLONES stress_vm_grayflip(
 	}
 	val++;
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("gray code (flip)", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -923,6 +946,8 @@ static size_t TARGET_CLONES stress_vm_incdec(
 			bit_errors++;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("incdec code", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -986,6 +1011,8 @@ static size_t TARGET_CLONES stress_vm_prime_incdec(
 			bit_errors++;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("prime-incdec", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -1052,6 +1079,8 @@ static size_t TARGET_CLONES stress_vm_swap(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	/* Reverse swaps */
 	for (i = chunks - 1, ptr = (uint8_t *)buf_end - chunk_sz; ptr >= (uint8_t *)buf; ptr -= chunk_sz, i--) {
 		size_t offset = swaps[i];
@@ -1074,12 +1103,14 @@ static size_t TARGET_CLONES stress_vm_swap(
 			goto abort;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
 	stress_mwc_set_seed(w1, z1);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
-		uint8_t *p = (uint8_t *)ptr;
+		const uint8_t *p = (uint8_t *)ptr;
 		const uint8_t *p_end = (uint8_t *)ptr + chunk_sz;
 		uint8_t val = stress_mwc8();
 
@@ -1091,6 +1122,8 @@ static size_t TARGET_CLONES stress_vm_swap(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("swap bytes", bit_errors);
 abort:
 	free(swaps);
@@ -1147,6 +1180,8 @@ static size_t TARGET_CLONES stress_vm_rand_set(
 			goto abort;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
@@ -1165,6 +1200,8 @@ static size_t TARGET_CLONES stress_vm_rand_set(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("rand-set", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -1221,6 +1258,8 @@ static size_t TARGET_CLONES stress_vm_ror(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
@@ -1245,6 +1284,8 @@ static size_t TARGET_CLONES stress_vm_ror(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 
 	inject_random_bit_errors(buf, sz);
@@ -1265,6 +1306,8 @@ static size_t TARGET_CLONES stress_vm_ror(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("ror", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -1329,6 +1372,8 @@ static size_t TARGET_CLONES stress_vm_flip(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 
 	for (i = 0; i < 8; i++) {
@@ -1359,6 +1404,8 @@ static size_t TARGET_CLONES stress_vm_flip(
 		(void)stress_mincore_touch_pages(buf, sz);
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
 	stress_mwc_set_seed(w, z);
@@ -1384,6 +1431,8 @@ static size_t TARGET_CLONES stress_vm_flip(
 			break;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("flip", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -1429,6 +1478,8 @@ static size_t TARGET_CLONES stress_vm_zero_one(
 	}
 
 	(void)shim_memset(buf, 0xff, sz);
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 	c += sz / 8;
@@ -1446,6 +1497,8 @@ static size_t TARGET_CLONES stress_vm_zero_one(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("zero-one", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -1488,6 +1541,8 @@ static size_t TARGET_CLONES stress_vm_galpat_zero(
 			}
 		}
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
@@ -1508,6 +1563,8 @@ static size_t TARGET_CLONES stress_vm_galpat_zero(
 	if (bits_set != bits_bad)
 		bit_errors += UNSIGNED_ABS(bits_set, bits_bad);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("galpat-zero", bit_errors);
 ret:
 	if (UNLIKELY(max_ops && (c >= max_ops)))
@@ -1552,6 +1609,8 @@ static size_t TARGET_CLONES stress_vm_galpat_one(
 			}
 		}
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
@@ -1572,6 +1631,8 @@ static size_t TARGET_CLONES stress_vm_galpat_one(
 	if (bits_set != bits_bad)
 		bit_errors += UNSIGNED_ABS(bits_set, bits_bad);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("galpat-one", bit_errors);
 ret:
 	if (UNLIKELY(max_ops && (c >= max_ops)))
@@ -1648,6 +1709,8 @@ static size_t TARGET_CLONES stress_vm_inc_nybble(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
@@ -1664,6 +1727,8 @@ static size_t TARGET_CLONES stress_vm_inc_nybble(
 			break;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("inc-nybble", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -1719,6 +1784,8 @@ static size_t TARGET_CLONES stress_vm_rand_sum(
 			goto abort;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
@@ -1735,6 +1802,8 @@ static size_t TARGET_CLONES stress_vm_rand_sum(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("rand-sum", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -1787,12 +1856,16 @@ static size_t TARGET_CLONES stress_vm_prime_zero(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
 	for (ptr = (uint8_t *)buf + offset; ptr < (uint8_t *)buf_end; ptr += prime)
 		bit_errors += stress_vm_count_bits8(*ptr);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("prime-zero", bit_errors);
 abort:
 	offset++;
@@ -1847,12 +1920,16 @@ static size_t TARGET_CLONES stress_vm_prime_one(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
 	for (ptr = (uint8_t *)buf + offset; ptr < (uint8_t *)buf_end; ptr += prime)
 		bit_errors += 8 - stress_vm_count_bits8(*ptr);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("prime-one", bit_errors);
 abort:
 	offset++;
@@ -1907,12 +1984,16 @@ static size_t TARGET_CLONES stress_vm_prime_gray_zero(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
 	for (ptr = (uint8_t *)buf + offset; ptr < (uint8_t *)buf_end; ptr += prime)
 		bit_errors += stress_vm_count_bits8(*ptr);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("prime-gray-zero", bit_errors);
 abort:
 	offset++;
@@ -1967,12 +2048,16 @@ static size_t TARGET_CLONES stress_vm_prime_gray_one(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
 	for (ptr = (uint8_t *)buf + offset; ptr < (uint8_t *)buf_end; ptr += prime)
 		bit_errors += 8 - stress_vm_count_bits8(*ptr);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("prime-gray-one", bit_errors);
 abort:
 	offset++;
@@ -2072,6 +2157,8 @@ static size_t OPTIMIZE3 TARGET_CLONES stress_vm_write64(
 		if (UNLIKELY(!stress_continue_flag() || (max_ops && (i >= max_ops))))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_bogo_add(args, i);
 	val++;
 
@@ -2142,6 +2229,8 @@ static size_t TARGET_CLONES stress_vm_write64nt(
 		}
 		stress_bogo_add(args, i);
 		val++;
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			stress_cpu_data_cache_flush(buf, sz);
 		return 0;
 	}
 	return stress_vm_write64(buf, buf_end, sz, args, max_ops);
@@ -2212,6 +2301,8 @@ static size_t OPTIMIZE3 TARGET_CLONES stress_vm_read64(
 			break;
 	}
 	stress_bogo_add(args, i);
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 
 	return 0;
 }
@@ -2269,6 +2360,8 @@ static size_t TARGET_CLONES stress_vm_write1024v(
 	}
 	stress_bogo_add(args, i);
 	val++;
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 
 	return 0;
 }
@@ -2366,6 +2459,8 @@ static size_t TARGET_CLONES stress_vm_rowhammer(
 	stress_bogo_add(args, VM_ROWHAMMER_LOOPS);
 	val = (val >> 31) | (val << 1);
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("rowhammer", bit_errors);
 
 	return bit_errors;
@@ -2383,7 +2478,8 @@ static size_t TARGET_CLONES stress_vm_mscan(
 	const uint64_t max_ops)
 {
 	size_t bit_errors = 0;
-	register uint8_t *ptr = (uint8_t *)buf, *end;
+	register uint8_t *ptr = (uint8_t *)buf;
+	register const uint8_t *end;
 	register uint64_t c = stress_bogo_get(args);
 
 	(void)sz;
@@ -2445,6 +2541,8 @@ static size_t TARGET_CLONES stress_vm_mscan(
 	}
 
 abort:
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("mscan", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -2602,6 +2700,8 @@ static size_t TARGET_CLONES stress_vm_cache_stripe(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += 64) {
 		bit_errors += (ptr[0x00] != 0xa0);
@@ -2670,6 +2770,8 @@ static size_t TARGET_CLONES stress_vm_cache_stripe(
 		bit_errors += (ptr[0x20] != 0x30);
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 	stress_vm_check("cache-stripe", bit_errors);
@@ -2701,6 +2803,8 @@ static size_t TARGET_CLONES stress_vm_cache_lines(
 		*ptr = i++;
 		stress_asm_mb();
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	c += sz / stress_vm_cache_line_size;
 	if (UNLIKELY(max_ops && (c >= max_ops)))
 		goto abort;
@@ -2712,6 +2816,8 @@ static size_t TARGET_CLONES stress_vm_cache_lines(
 	}
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("cache-lines", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -2789,6 +2895,8 @@ static size_t TARGET_CLONES stress_vm_wrrd128nt(
 		val++;
 		c++;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	if (UNLIKELY(max_ops && (c >= max_ops)))
 		goto abort;
 	if (UNLIKELY(!stress_continue_flag()))
@@ -2834,6 +2942,8 @@ static size_t TARGET_CLONES stress_vm_wrrd128nt(
 		bit_errors += (tmp != (val + 3));
 		val++;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	inject_random_bit_errors(buf, sz);
 	stress_vm_check("wrrd128nt", bit_errors);
 abort:
@@ -2902,6 +3012,8 @@ static size_t TARGET_CLONES stress_vm_fwdrev(
 		if (UNLIKELY(!stress_continue_flag()))
 			goto abort;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 
 	for (fwdptr = (uint8_t *)buf, revptr = (uint8_t *)buf_end; fwdptr < (uint8_t *)buf_end; ) {
 		bit_errors += (*(fwdptr + 0) != ((rnd >> 0x00) & 0xff));
@@ -2945,6 +3057,8 @@ static size_t TARGET_CLONES stress_vm_fwdrev(
 			goto abort;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("fwdrev", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -3004,6 +3118,8 @@ static size_t stress_vm_lfsr32(
 			goto abort;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
@@ -3027,6 +3143,8 @@ static size_t stress_vm_lfsr32(
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("lfsr32", bit_errors);
 abort:
 	stress_bogo_set(args, c);
@@ -3120,6 +3238,8 @@ static size_t TARGET_CLONES stress_vm_checkerboard(
 		stress_asm_mb();
 		STRESS_VM_CHECKERBOARD_SWAP(ptr + 6, ptr + 7);
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
@@ -3139,6 +3259,8 @@ static size_t TARGET_CLONES stress_vm_checkerboard(
 			break;
 	}
 
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(buf, sz);
 	stress_vm_check("checkerboard", bit_errors);
 	stress_bogo_set(args, c);
 
@@ -3268,7 +3390,7 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			vm_bytes = MIN_VM_BYTES;
 	}
-	vm_bytes /= args->num_instances;
+	vm_bytes /= args->instances;
 	if (vm_bytes < MIN_VM_BYTES)
 		vm_bytes = MIN_VM_BYTES;
 	buf_sz = vm_bytes & ~(page_size - 1);
@@ -3444,7 +3566,7 @@ static int stress_vm(stress_args_t *args)
 	if (args->instance == 0)
 		pr_dbg("%s: using method '%s'\n", args->name, context.vm_method->name);
 
-	for (retries = 0; (retries < 100) && stress_continue_flag(); retries++) {
+	for (retries = 0; LIKELY((retries < 100) && stress_continue_flag()); retries++) {
 		context.bit_error_count = (uint64_t *)
 			stress_mmap_populate(NULL, page_size,
 				PROT_READ | PROT_WRITE,
@@ -3457,11 +3579,15 @@ static int stress_vm(stress_args_t *args)
 
 	/* Cannot allocate a single page for bit error counter */
 	if (context.bit_error_count == MAP_FAILED) {
-		if (stress_continue_flag()) {
+		if (LIKELY(stress_continue_flag())) {
 			pr_err("%s: could not mmap bit error counter: "
 				"retry count=%zu, errno=%d (%s)\n",
 				args->name, retries, err, strerror(err));
 		}
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+		if (context.numa_mask)
+			stress_numa_mask_free(context.numa_mask);
+#endif
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(context.bit_error_count, page_size, "bit-error-count");
