@@ -20,6 +20,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-cpu-cache.h"
 #include "core-killpid.h"
 #include "core-out-of-memory.h"
 #include "core-pthread.h"
@@ -159,7 +160,6 @@ static void *stress_mmapaddr_get_addr(stress_args_t *args)
 	free(addr);
 
 	while (stress_vma_continue_flag && stress_vma_continue(args)) {
-		ssize_t ret;
 		uintptr_t test_addr;
 
 		if (sizeof(uintptr_t) > 4) {
@@ -184,6 +184,7 @@ static void *stress_mmapaddr_get_addr(stress_args_t *args)
 
 		for (i = 0, test_addr = (uintptr_t)addr; i < STRESS_VMA_PAGES; i++, test_addr += page_size) {
 			int fd[2], err;
+			ssize_t ret;
 
 			if (UNLIKELY(pipe(fd) < 0))
 				return NULL;
@@ -241,7 +242,7 @@ static void *stress_vma_mmap(void *ptr)
 		const size_t offset = page_size * stress_mwc8modn(STRESS_VMA_PAGES);
 		const int prot = prots[stress_mwc8modn(SIZEOF_ARRAY(prots))];
 		int flags = MAP_FIXED | MAP_ANONYMOUS;
-		void *mapped;
+		const void *mapped;
 
 		flags |= (stress_mwc1() ? MAP_SHARED : MAP_PRIVATE);
 #if defined(MAP_GROWSDOWN)
@@ -355,6 +356,7 @@ static void *stress_vma_madvise(void *ptr)
 	stress_args_t *args = (stress_args_t *)ctxt->args;
 	const uintptr_t data = (uintptr_t)ctxt->data;
 	const size_t page_size = args->page_size;
+	const bool aggressive = !!(g_opt_flags & OPT_FLAGS_AGGRESSIVE);
 
 	static const int advice[] = {
 #if defined(MADV_NORMAL)
@@ -402,6 +404,8 @@ static void *stress_vma_madvise(void *ptr)
 
 		if (madvise((void *)(data + offset), len, advice[i]) == 0)
 			stress_vma_metrics->s.metrics[STRESS_VMA_MADVISE]++;
+		if (aggressive)
+			stress_cpu_data_cache_flush((void *)ptr, page_size);
 	}
 	return NULL;
 }
@@ -542,13 +546,16 @@ static void *stress_vma_access(void *ptr)
 	stress_args_t *args = (stress_args_t *)ctxt->args;
 	const uintptr_t data = (uintptr_t)ctxt->data;
 	const size_t page_size = args->page_size;
+	const bool aggressive = !!(g_opt_flags & OPT_FLAGS_AGGRESSIVE);
 
 	while (stress_vma_continue_flag && stress_vma_continue(args)) {
 		const size_t offset = page_size * stress_mwc8modn(STRESS_VMA_PAGES);
-		volatile uint8_t *ptr8 = (volatile uint8_t *)(data + offset);
+		volatile const uint8_t *ptr8 = (volatile uint8_t *)(data + offset);
 
 		stress_vma_metrics->s.metrics[STRESS_VMA_ACCESS]++;
 		stress_uint8_put(*ptr8);
+		if (aggressive)
+			stress_cpu_data_cache_flush((void *)ptr, page_size);
 	}
 	return NULL;
 }
@@ -674,7 +681,7 @@ static int stress_vma_child(stress_args_t *args, void *void_ctxt)
 	for (i = 0; i < STRESS_VMA_PROCS; i++)
 		stress_sync_start_init(&s_pids[i]);
 
-	for (i = 0; (stress_continue(args)) && (i < STRESS_VMA_PROCS); i++) {
+	for (i = 0; LIKELY(stress_continue(args) && (i < STRESS_VMA_PROCS)); i++) {
 		s_pids[i].pid = fork();
 		if (s_pids[i].pid < 0)
 			continue;
